@@ -40,6 +40,7 @@ static void usage(void)
     printf(EXEC_NAME " Copyright (C) 2008-2010, 2012, 2013, 2017, 2019-20 "
            "Matt Davis (enferex)\n"
            "Special thanks to all contributors and the 757 crew.\n"
+           "See the AUTHORS file for a list of other contributors.\n"
            "This program comes with ABSOLUTELY NO WARRANTY\n"
            "This is free software, and you are welcome to redistribute it\n"
            "under certain conditions.  For details see the file 'LICENSE'\n"
@@ -47,11 +48,12 @@ static void usage(void)
            "<http://www.gnu.org/licenses/gpl-3.0.txt>\n\n");
     
     printf("-- " EXEC_NAME " v" VER" --\n"
-           "Usage: ./" EXEC_NAME " <file.pdf> [-i] [-w] [-q] [-s]\n"
+           "Usage: ./" EXEC_NAME " <file.pdf> [-i] [-w] [-q]\n"
            "\t -i Display PDF creator information\n"
            "\t -w Write the PDF versions and summary to disk\n"
-           "\t -q Display only the number of versions contained in the PDF\n"
-           "\t -s Scrub the previous history data from the specified PDF\n");
+           "\t -q Display only the number of versions contained in the PDF\n");
+// Experimental feature:
+//           "\t -s Scrub the previous history data from the specified PDF\n");
 
     exit(0);
 }
@@ -72,8 +74,8 @@ static void write_version(
     /* Create file */
     if ((c = strstr(fname, ".pdf")))
       *c = '\0';
-    new_fname = safe_calloc(strlen(fname) + strlen(dirname) + 16);
-    snprintf(new_fname, strlen(fname) + strlen(dirname) + 16,
+    new_fname = safe_calloc(strlen(fname) + strlen(dirname) + 32);
+    snprintf(new_fname, strlen(fname) + strlen(dirname) + 32,
              "%s/%s-version-%d.pdf", dirname, fname, xref->version);
 
     if (!(new_fp = fopen(new_fname, "w")))
@@ -99,6 +101,7 @@ static void write_version(
 }
 
 
+#ifdef PDFRESURRECT_EXPERIMENTAL
 static void scrub_document(FILE *fp, const pdf_t *pdf)
 {
     FILE *new_fp;
@@ -106,6 +109,9 @@ static void scrub_document(FILE *fp, const pdf_t *pdf)
     char *new_name, *c;
     const char *suffix = "-scrubbed.pdf";
 
+    printf("The scrub feature (-s) is experimental and likely not to work as "
+           "expected.\n");
+
     /* Create a new name */
     if (!(new_name = malloc(strlen(pdf->name) + strlen(suffix) + 1)))
     {
@@ -177,6 +183,7 @@ static void scrub_document(FILE *fp, const pdf_t *pdf)
     free(new_name);
     fclose(new_fp);
 }
+#endif // PDFRESURRECT_EXPERIMENTAL
 
 
 static void display_creator(FILE *fp, const pdf_t *pdf)
@@ -207,7 +214,6 @@ static pdf_t *init_pdf(FILE *fp, const char *name)
       pdf_delete(pdf);
       return NULL;
     }
-    pdf_load_pages_kids(fp, pdf);
 
     return pdf;
 }
@@ -253,8 +259,10 @@ int main(int argc, char **argv)
           flags |= PDF_FLAG_DISP_CREATOR;
         else if (strncmp(argv[i], "-q", 2) == 0)
           flags |= PDF_FLAG_QUIET;
+#ifdef PDFRESURRECT_EXPERIMENTAL
         else if (strncmp(argv[i], "-s", 2) == 0)
           do_scrub = 1;
+#endif
         else if (argv[i][0] != '-')
           name = argv[i];
         else if (argv[i][0] == '-')
@@ -336,9 +344,11 @@ int main(int argc, char **argv)
     /* Generate a per-object summary */
     pdf_summarize(fp, pdf, dname, flags);
 
+#ifdef PDFRESURRECT_EXPERIMENTAL
     /* Have we been summoned to scrub history from this PDF */
     if (do_scrub)
       scrub_document(fp, pdf);
+#endif
 
     /* Display extra information */
     if (flags & PDF_FLAG_DISP_CREATOR)
