@@ -118,9 +118,6 @@ static char *get_object(
     size_t       *size,
     int          *is_stream);
 
-static void add_kid(int id, xref_t *xref);
-static void load_kids(FILE *fp, int pages_id, xref_t *xref);
-
 static const char *get_type(FILE *fp, int obj_id, const xref_t *xref);
 /* static int get_page(int obj_id, const xref_t *xref); */
 static char *get_header(FILE *fp);
@@ -169,7 +166,6 @@ void pdf_delete(pdf_t *pdf)
     {
         free(pdf->xrefs[i].creator);
         free(pdf->xrefs[i].entries);
-        free(pdf->xrefs[i].kids);
     }
 
     free(pdf->name);
@@ -180,16 +176,13 @@ void pdf_delete(pdf_t *pdf)
 
 int pdf_is_pdf(FILE *fp)
 {
-    int   is_pdf;
     char *header;
+    if (!(header = get_header(fp)))
+      return 0;
 
-    header = get_header(fp);
-
-    if (header && strstr(header, "%PDF-"))
-      is_pdf = 1;
-    else 
-      is_pdf = 0;
-
+    /* First 1024 bytes of doc must be header (1.7 spec pg 1102) */
+    const char *c = strstr(header, "%PDF-");
+    const int is_pdf = c && ((c - header+strlen("%PDF-M.m")) < 1024);
     free(header);
     return is_pdf;
 }
@@ -197,13 +190,16 @@ int pdf_is_pdf(FILE *fp)
 
 void pdf_get_version(FILE *fp, pdf_t *pdf)
 {
-    char *header, *c;
+    char *header = get_header(fp);
 
-    header = get_header(fp);
-
-    /* Locate version string start and make sure we dont go past header */
+    /* Locate version string start and make sure we dont go past header
+     * The format is %PDF-M.m, where 'M' is the major number and 'm' minor.
+     */
+    const char *c;
     if ((c = strstr(header, "%PDF-")) && 
-        (c + strlen("%PDF-M.m") + 2))
+        ((c + 6)[0] == '.') && // Separator
+        isdigit((c + 5)[0]) && // Major number
+        isdigit((c + 7)[0]))   // Minor number
     {
         pdf->pdf_major_version = atoi(c + strlen("%PDF-"));
         pdf->pdf_minor_version = atoi(c + strlen("%PDF-M."));
@@ -310,55 +306,6 @@ int pdf_load_xrefs(FILE *fp, pdf_t *pdf)
 
 
 /* Load page information */
-void pdf_load_pages_kids(FILE *fp, pdf_t *pdf)
-{
-    int     i, id, dummy;
-    char   *buf, *c;
-    long    start, sz;
-
-    start = ftell(fp);
-
-    /* Load all kids for all xref tables (versions) */
-    for (i=0; i<pdf->n_xrefs; i++)
-    {
-        if (pdf->xrefs[i].version && (pdf->xrefs[i].end != 0))
-        {
-            fseek(fp, pdf->xrefs[i].start, SEEK_SET);
-            while (SAFE_F(fp, (fgetc(fp) != 't')))
-                ; /* Iterate to trailer */
-
-            /* Get root catalog */
-            sz = pdf->xrefs[i].end - ftell(fp);
-            buf = safe_calloc(sz + 1);
-            SAFE_E(fread(buf, 1, sz, fp), sz, "Failed to load /Root.\n");
-            buf[sz] = '\0';
-            if (!(c = strstr(buf, "/Root")))
-            {
-                free(buf);
-                continue;
-            }
-
-            /* Jump to catalog (root) */
-            id = atoi(c + strlen("/Root") + 1);
-            free(buf);
-            buf = get_object(fp, id, &pdf->xrefs[i], NULL, &dummy);
-            if (!buf || !(c = strstr(buf, "/Pages")))
-            {
-                free(buf);
-                continue;
-            }
-
-            /* Start at the first Pages obj and get kids */
-            id = atoi(c + strlen("/Pages") + 1);
-            load_kids(fp, id, &pdf->xrefs[i]);
-            free(buf); 
-        }
-    }
-            
-    fseek(fp, start, SEEK_SET);
-}
-
-
 char pdf_get_object_status(
     const pdf_t *pdf,
     int          xref_idx,
@@ -641,9 +588,10 @@ static void load_xref_entries(FILE *fp, xref_t *xref)
 
 static void load_xref_from_plaintext(FILE *fp, xref_t *xref)
 {
-    int  i, buf_idx, obj_id, added_entries;
+    int  i, obj_id, added_entries;
     char c, buf[32] = {0};
     long start, pos;
+    size_t buf_idx;
 
     start = ftell(fp);
 
@@ -692,9 +640,20 @@ static void load_xref_from_plaintext(FILE *fp, xref_t *xref)
         /* Entry or object id */
         if (strlen(buf) > 17)
         {
+            const char *token = NULL;
             xref->entries[i].obj_id = obj_id++;
-            xref->entries[i].offset = atol(strtok(buf, " "));
-            xref->entries[i].gen_num = atoi(strtok(NULL, " "));
+            token = strtok(buf, " ");
+            if (!token) {
+              FAIL("Failed to parse xref entry. "
+                   "This might be a corrupt PDF.\n");
+            }
+            xref->entries[i].offset = atol(token);
+            token = strtok(NULL, " ");
+            if (!token) {
+              FAIL("Failed to parse xref entry. "
+                   "This might be a corrupt PDF.\n");
+            }
+            xref->entries[i].gen_num = atoi(token);
             xref->entries[i].f_or_n = buf[17];
             ++added_entries;
         }
@@ -1055,12 +1014,13 @@ static void load_creator_from_old_format(
     } /* For all creation information tags */
 
     /* Go through the values and convert if encoded */
-    for (i=0; i<n_eles; ++i)
-      if ((ascii = decode_text_string(info[i].value, strlen(info[i].value))))
-      {
-          strncpy(info[i].value, ascii, strlen(info[i].value));
-          free(ascii);
+    for (i = 0; i < n_eles; ++i) {
+      const size_t val_str_len = strnlen(info[i].value, KV_MAX_VALUE_LENGTH);
+      if ((ascii = decode_text_string(info[i].value, val_str_len))) {
+        strncpy(info[i].value, ascii, val_str_len);
+        free(ascii);
       }
+    }
 
     xref->creator = info;
     xref->n_creator_entries = n_eles;
@@ -1169,7 +1129,7 @@ static char *get_object(
         if ((c = strstr(data + search, "endobj")))
         {
             *(c + strlen("endobj") + 1) = '\0';
-            obj_sz = (void *)strstr(data + search, "endobj") - (void *)data;
+            obj_sz = (char *)strstr(data + search, "endobj") - (char *)data;
             obj_sz += strlen("endobj") + 1;
             break;
         }
@@ -1195,56 +1155,6 @@ static char *get_object(
 }
 
 
-static void add_kid(int id, xref_t *xref)
-{
-    /* Make some space */
-    if (((xref->n_kids + 1) * KID_SIZE) > (xref->n_kids_allocs*KIDS_PER_ALLOC))
-      xref->kids = realloc(
-          xref->kids, (++xref->n_kids_allocs)*(KIDS_PER_ALLOC * KID_SIZE));
-
-    xref->kids[xref->n_kids++] = id;
-}
-
-
-/* Recursive */
-static void load_kids(FILE *fp, int pages_id, xref_t *xref)
-{
-    int   dummy, buf_idx, kid_id;
-    char *data, *c, buf[32];
-
-    /* Get kids */
-    data = get_object(fp, pages_id, xref, NULL, &dummy);
-    if (!data || !(c = strstr(data, "/Kids")))
-    {
-        free(data);
-        return;
-    }
-    
-    c = strchr(c, '[');
-    buf_idx = 0;
-    memset(buf, 0, sizeof(buf));
-    while (*(++c) != ']')
-    {
-        if (isdigit(*c) || (*c == ' '))
-          buf[buf_idx++] = *c;
-        else if (isalpha(*c))
-        {
-            kid_id = atoi(buf);
-            add_kid(kid_id, xref);
-            buf_idx = 0;
-            memset(buf, 0, sizeof(buf));
-
-            /* Check kids of kid */
-            load_kids(fp, kid_id, xref);
-        }
-        else if (*c == ']')
-          break;
-    }
-
-    free(data);
-}
-
-
 static const char *get_type(FILE *fp, int obj_id, const xref_t *xref)
 {
     int          is_stream;
@@ -1290,34 +1200,30 @@ static const char *get_type(FILE *fp, int obj_id, const xref_t *xref)
     while (isspace(*c) || *c == '/')
       ++c;
 
-    /* Return the value by storing it in static mem */
-    memcpy(buf, c, (((c - obj) < sizeof(buf)) ? c - obj : sizeof(buf)));
-    c = buf;
-    while (!(isspace(*c) || *c=='/' || *c=='>'))
-      ++c;
-    *c = '\0';
+    /* 'c' should be pointing to the type name.  Find the end of the name. */
+    size_t n_chars = 0;
+    const char *name_itr = c;
+    while ((name_itr < endobj) &&
+           !(isspace(*name_itr) || *name_itr == '/' || *name_itr == '>')) {
+        ++name_itr;
+        ++n_chars;
+    }
+    if (n_chars >= sizeof(buf))
+    {
+        free(obj);
+        fseek(fp, start, SEEK_SET);
+        return "Unknown";
+    }
 
+    /* Return the value by storing it in static mem. */
+    memcpy(buf, c, n_chars);
+    buf[n_chars] = '\0';
     free(obj);
     fseek(fp, start, SEEK_SET);
-
     return buf;
 }
 
 
-/* TODO
-static int get_page(int obj_id, const xref_t *xref)
-{
-    int i;
-
-    for (i=0; i<xref->n_kids; i++)
-      if (xref->kids[i] == obj_id)
-        break;
-
-    return i;
-}
-*/
-
-
 static char *get_header(FILE *fp)
 {
     /* First 1024 bytes of doc must be header (1.7 spec pg 1102) */
@@ -1340,8 +1246,8 @@ static char *decode_text_string(const char *str, size_t str_len)
     /* Regular encoding */
     if (str[0] == '(')
     {
-        ascii = safe_calloc(strlen(str) + 1);
-        strncpy(ascii, str, strlen(str) + 1);
+        ascii = safe_calloc(str_len + 1);
+        strncpy(ascii, str, str_len + 1);
         return ascii;
     }
     else if (str[0] == '<')
